require "dream/version"
require 'json'

module Dream
  class Spectre
    def initialize(library)
      @library = library
      @messages = library['records']['messages']
      @format = library['records']['format']
      @watches = {}
      @book = Array.new
    end

    def record(status, directory, message)
      timestamp = Time.now
      path = directory.split("/")
      node = branch(path[0..-2])
      record = timeline(path[-1], timestamp, status, message)
      node.push(record)
    end

    def record_start()
      watch(@messages['started'])
    end

    def record_finished()
      watch(@messages['finished'])
    end

    def watch(name)
      @watches[:"#{name}"] = timestamp()
    end

    def time_records(name)
      return @watches[:"#{name}"]
    end

    def started
      return @messages['started']
    end

    def finished
      return @messages['finished']
    end

    def success
      return @messages['success']
    end

    def info
      return @messages['info']
    end

    def warning
      return @messages['warning']
    end

    def error
      return @messages['error']
    end

    def manuscript(clear)
      replica = @book
      if clear
        @book = Array.new
      end
      return replica
    end

    def author(book)
      @book = book
    end

    def save(path)
      path = path.eql?(nil) ? '.' : path
      iteration = 0
      name = record_name(iteration)
      record_path = "#{path}/#{name}"
      while File.exist?(record_path)
        iteration = iteration + 1
        name = record_name(iteration)
        record_path = "#{path}/#{name}"
      end
      File.open(record_path, "w+") do |f|
        f.puts(@book.to_json)
      end
      return record_path
    end

    def clean(path)
      return FileUtils.rm_rf(path)
    end

    private

    def timeline(name, timestamp, status, message)
      record = {
        "name" => name,
        "timestamp" => timestamp,
        "status" => status,
        "message" => message,
        "children" => []
      }
      return record
    end

    def branch(branches)
      edge = @book
      branches.each do | branch |
        if edge[-1]["name"].eql?(branch)
          edge = edge[-1]["children"]
        else
          break
        end
      end
      return edge
    end

    def record_name(iteration)
      timestamp = @library['records']['timestamp'] ? "-#{Time.now.strftime "%Y%m%d%H%M%S-%L"}" : ""
      if iteration.eql?(0)
        return "#{@library['records']['name']}#{timestamp}.#{@library['records']['extension']}"
      else
        return "#{@library['records']['name']}#{timestamp}-#{iteration}.#{@library['records']['extension']}"
      end
    end

    def timestamp()
      return Time.now.strftime "%Y-%m-%dT%H:%M:%S.%LZ"
    end


  end
end
